<?php

// $Id$

/**
 * @file
 *  Link general private message module functionalities to services module.
 */

/**
 * Get all of the logged in user's private messages.
 *
 * @param <string> $type
 * Which type of messages you would like to retrieve
 * Possibilities are inbox or sent.
 *
 * @param <bool> $load_full
 * Boolean. Whether to load the full message for each message or not.
 *
 * @param $offset
 *   Integer (optional). An offset integer for paging.
 * @param $limit
 *   Integer (optional). A limit integer for paging.
 *
 * @return
 *   An array of messages.
 *
 */
function pm_service_get($type = 'inbox', $load_full = FALSE, $offset = 0, $limit = 0, $uid = '') {

  $messages = array();
  $pms = array();

  // If no type passed set default type to inbox.
  if (!$type) {
    $type = 'inbox';
  }
  elseif ($type != 'inbox' && $type != 'sent') {
    return services_error(t('Invalid type passed'), 400);
  }
  
  // User needs to be authenticated to proceed
  global $user;
  if (!user_is_logged_in()) {
    return services_error(t('This user is not logged in.'), 403);
  }
    
  // If a user id other than the current user's ID is passed,
  // validate that the authenticated user has the correct 
  // permissions to read another user's messages.
  if (is_numeric($uid) && ($uid != $user->uid)) {
    if (user_access("read all private messages")) {
      $account = user_load($uid);
    }
    else {
      return services_error(t('This user does not have permissions to read all messages.'), 403);
    }
  }
  // Load the full account object of the current user
  else {
    $account = user_load($user->uid);
  }

  module_load_include('inc', 'privatemsg', 'privatemsg.pages');
  // Construct the query and retrieve the correct set of messages.
  $query = _privatemsg_assemble_query('list', $account, $type);
  
  // Handle any paging if an offset or a limit was passed in.
  if (empty($offset) && !empty($limit)) {
    $result = db_query_range($query['query'], 0, $limit);
  }
  elseif (!empty($offset) && !empty($limit)) {
    $result = db_query_range($query['query'], $offset, $limit);
  }
  elseif (!empty($offset) && empty($limit)) {
    $rows = (int)db_result(db_query($query['count']));
    $result = db_query_range($query['query'], $offset, $rows);
  }
  else {
    // No paging, we are retrieving everything.
    $result = db_query($query['query']);
  }
  
  // Loop the result object to get the messages.
  while ($row = db_fetch_object($result)) {
    $pms[$row->thread_id] = $row;
  }
  
  // Check whether we are loading the full thread or not.
  if (!empty($pms) && $load_full) {
    foreach ($pms as $thread_id => $msg) {
      $messages[] = privatemsg_thread_load($thread_id, $account, NULL, FALSE);
    }
  }
  else {
    $messages = $pms;
  }
  
  // Return messages.
  return $messages;
  
}

/**
 * Get the number of unread private messages of the logged-in user.
 *
 * @return
 *   The unread count.
 */
function pm_service_unread_count() {

  global $user;
  $account = user_load($user->uid);
  
  // User needs to be authenticated to proceed
  if (!user_is_logged_in()) {
    return services_error(t('The user is not logged in.'), 403);
  }
  // return unread count.
  else {
    return privatemsg_unread_count($account);
  }

}

/**
 * Send a private message to one or more recipients.
 *
 * @param <string> $recipients
 * A comma separated list of usernames for recipients of the message.
 *
 * @param <string> $subject
 * A message subject
 *
 * @param <string> $body
 * A message body
 *
 * @param <int> $thread_id
 * A thread ID. pass this parameter if you are sending a message reply.
 *
 * @return <bool>
 * Return TRUE if sending the message was successful.
 */
function pm_service_send($recipients, $subject, $body = '', $thread_id = NULL) {

  module_load_include('inc', 'privatemsg', 'privatemsg.pages');
  $form_state = array();
  
  // Make sure the message author is logged in
  global $user;
  $account = user_load($user->uid);
  
  if (!user_is_logged_in()) {
    return services_error(t('Author is not logged in.'), 403);
  }
  
  // Validate at least 1 recipient has been passed in.
  if ($recipients == '' && !$thread_id) {
    return services_error(t('There are no recipients, please enter a recipient for the message.'), 400);
  }
  elseif (!$thread_id) {
    // No thread ID - we are sending a new message.
    // Convert the recipients string to an array of user objects.
    list($recipients, $invalid) = _privatemsg_parse_userstring($recipients);
    $result = privatemsg_new_thread($recipients, $subject, $body, array('author' => $account));
    if ($result['success']) {
      return TRUE;
    }
    else {
      return services_error(implode("\n", $result['messages']['error']), 400);
    }
  }
  else {
    // There is a thread id so we are sending a reply.
    $result = privatemsg_reply($thread_id, $body, array('author' => $account));
    if ($result['success']) {
      return TRUE;
    }
    elseif (!empty($result[0])) {
      // If $result[0] this means the thread could not be loaded.
      return services_error($result[0], 404);
    }
    else {
      // else there was some other problem.
      return services_error(implode("\n", $result['messages']['error']), 400);
    }
  }
  
}

/**
 * Get all messages in a thread.
 *
 * @param <int> @thread_id
 *   ID of the thread to be loaded.
 * @param <int> $offset
 *   Optional: Message offset from the start of the thread.
 * @return
 *   An array of messages in a thread.
 */
function pm_service_get_thread($thread_id, $offset = 0) {

  // Return if wrong paramters are passed
  if (!$thread_id || !is_numeric($thread_id)) {
    return services_error(t('Invalid parameters passed'), 400);
  }

  // Make sure the user is logged in
  global $user;
  $account = user_load($user->uid);
  if (!user_is_logged_in()) {
    return services_error(t('The user is not logged in.'), 403);
  }

  // Return the full thread
  return privatemsg_thread_load($thread_id, $account, $offset);

}